package okapi;

import java.util.LinkedList;

/**
 * A simple efficient system for generating and maintaining cookies for any
 * system that requires identification. Cookie managers are <i>not</i> 
 * thread-safe (yet).
 * @author Sean Kelly
 */
public class CookieManager {
    private LinkedList<Integer> unused;
    private int counter;

    public CookieManager()
    {
        unused = new LinkedList<Integer>();
        counter = 0;
    }

    /**
     * Returns a distinct cookie to the caller. May be an as-of-yet unused
     * cookie or one that has been used and then relinquished to the manager.
     * @return 
     */
    public int newCookie()
    {
        int cookie;
        if (unused.isEmpty())
            // Generate a delicious fresh cookie
            cookie = counter++;
        else
            // Give somebody a used cookie. Eww.
            cookie = unused.pop();
        return cookie;
    }
    
    /**
     * Allows the cookie manager to reuse the cookie that has been released.
     * @param cookie The cookie that can be reused.
     */
    public void releaseCookie(int cookie)
    {
        unused.add(cookie);
    }
}
