package okapi;

import java.net.Socket;
import okapi.net.protocol.OkapiNetMessage;
import okapi.net.protocol.ProcessResponse;

/**
 * An OkapiNetWrapper is used to wrap incoming OkapiProcesses so that they can
 * be scheduled on a different scheduler without ruining pid association. Okapi
 * Wrappers accept Socket connections to networked nodes so that they can return
 * process to those nodes when completed.
 * @author Sean Kelly
 */
public class OkapiNetWrapper extends OkapiProcess {
    private OkapiProcess proc; // Process to execute
    private Socket retConnection; // Server connection
    
    /**
     * Creates a new OkapiNetWrapper that schedules the toDo task on the specified
     * scheduler, and returns the results as a ProcessResponse on the given
     * socket.
     * @param sched The scheduler that the wrapper will be scheduled to.
     * @param toDo The process to wrap and execute.
     * @param conn The connection on which to return the completed process as
     * a ProcessResponse.
     */
    public OkapiNetWrapper(Scheduler sched, OkapiProcess toDo, Socket conn)
    {
        super(sched, OkaProcessDisposal.DISPOSE);
        proc = toDo;

        retConnection = conn;
    }

    @Override
    public void run() {
        // Run input process
        proc.run();
        // Return process to server
        ProcessResponse retProc = new ProcessResponse();
        retProc.finished = proc;
        OkapiNetMessage.sendMessage(retProc, retConnection);
    }
}
