package okapi.deploy;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import okapi.SlaveNode;
import okapi.net.protocol.OkapiMessageType;
import okapi.net.protocol.OkapiNetMessage;
import okapi.net.protocol.SchedulerPing;

/**
 * The OkapiNode was created to allow an incredibly easy method for
 * setting up a working node in a cluster.
 * @author Sean Kelly
 */
public class OkapiNode {
    public static void run(int pingPort) throws IOException
    {
        boolean connected = false;
        int reconnectPort = 0;
        InetAddress reconnectAddr = null;
        
        while(!connected)
        {
            // Wait for a SchedulerPing from somewhere
            ServerSocket listenSock = new ServerSocket(pingPort);
            Socket pingSock = listenSock.accept();

            // Close the serving socket
            listenSock.close();

            // Handle the incoming connection
            OkapiNetMessage incoming;
            incoming = OkapiNetMessage.receiveMessage(pingSock);
            // If it is a SchedulerPing message, construct a connection
            if (incoming.type == OkapiMessageType.SCHEDULERPING)
            {
                SchedulerPing sp = (SchedulerPing) incoming;
                reconnectPort = sp.port;
                reconnectAddr = pingSock.getInetAddress();
                connected = true;
            }
            // Close the connected socket
            pingSock.close();
        }
        // Set up a connected SlaveNode with the given port and connect address
        SlaveNode workerNode = new SlaveNode(reconnectAddr, reconnectPort);
        workerNode.setDaemon(false); // Not sure if this is necessary
        workerNode.run(); // Call run directly, end when this ends.
    }
}
