package okapi.net.protocol;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.net.Socket;

/**
 * An OkapiNetMessage is the base type for any class implementing the Okapi
 * network protocol. It contains an OkapiMessageType so that recipients can
 * determine which sort of message they have received.
 * @author Sean Kelly
 */
public abstract class OkapiNetMessage implements Serializable {
   public OkapiMessageType type; // Type of net message being sent
   
   /**
    * Sends an OkapiNetMessage through the given Socket. Returns whether or not
    * it was successful.
    * @param toSend The message to send.
    * @param sender The socket to send the message through.
    * @return true if successful, false otherwise.
    */
   public static boolean sendMessage(OkapiNetMessage toSend, Socket sender)
   {
       boolean success = true;
       
       try {
           ObjectOutputStream out;
           out = new ObjectOutputStream(sender.getOutputStream());
           out.writeObject(toSend);
       } catch (IOException ioe) {
           success = false;
       } 
       return success;
   }
   
   /**
    * Receives an OkapiNetMessage from the given socket. Returns either the
    * received message or null if some error occurred.
    * @param receive The socket on which to listen for the next message
    * @return The message received from the given socket, or null if some
    * error occurred.
    */
   public static OkapiNetMessage receiveMessage(Socket receive)
   {
       OkapiNetMessage received = null;
       ObjectInputStream in;
       try {
           in = new ObjectInputStream(receive.getInputStream());
           received = (OkapiNetMessage) in.readObject();
       }
       catch (Exception e) {
           received = null;
           e.printStackTrace();
       }
       
       return received;
   } 
   
   // TODO receiveMessage with timeouts
}
